#ifndef _PUSSIANCE4_h_
#define _PUSSIANCE4_h_

#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include "src/game/game.h"
#include "src/board/board.h"
#include "src/computer/computer.h"

#define AI_SEED 0
#define MIN_COLUMN 4
#define MIN_ROW 4

/**
 * @brief Validates the command-line arguments provided by the user.
 *
 * @param argv The array of command-line arguments.
 * @param game_mode Pointer to store the selected game mode.
 * @param row_count Pointer to store the number of rows for the board.
 * @param col_count Pointer to store the number of columns for the board.
 * @return true if all arguments are valid, false otherwise.
 */
bool validate_arguments(char const* argv[], int* game_mode, int* row_count, int* col_count);

/**
 * @brief Announces the winner of the game based on the game mode and current turn.
 *
 * @param game_mode The game mode being played (e.g., player vs AI).
 * @param current_turn The player who made the last move.
 */
void announce_winner(enum GAME_MODE game_mode, int current_turn);

/**
 * @brief Checks if the provided game mode is valid.
 *
 * @param game_mode The game mode to validate.
 * @return true if the game mode is valid, false otherwise.
 */
bool is_valid_game_mode(int game_mode);

/**
 * @brief Checks if the provided board size is valid.
 *
 * @param rows The number of rows.
 * @param cols The number of columns.
 * @return true if the board size is valid, false otherwise.
 */
bool is_valid_board_size(int rows, int cols);

#endif
