#ifndef _BOARD_H_
#define _BOARD_H_

#include <stdlib.h>
#include <stdio.h>

enum BOARD_STATE {
    EMPTY = 0,
    PLAYER1 = 1,
    PLAYER2 = -1
};

/**
 * @brief Creates a new game board with the specified dimensions.
 *
 * @param rows The number of rows in the board.
 * @param cols The number of columns in the board.
 * @return A pointer to the newly allocated board (2D array).
 */
int** create_board(int rows, int cols);

/**
 * @brief Initializes the board by setting all cells to EMPTY.
 *
 * @param rows The number of rows in the board.
 * @param cols The number of columns in the board.
 * @param board The game board to initialize.
 */
void fill_board(int rows, int cols, int** board);

/**
 * @brief Prints the current state of the board to the console.
 *
 * @param rows The number of rows in the board.
 * @param cols The number of columns in the board.
 * @param board The game board to print.
 */
void print_board(int rows, int cols, int** board);

/**
 * @brief Frees the memory allocated for the board.
 *
 * @param rows The number of rows in the board.
 * @param board The board to destroy.
 * @return 0 on success, or -1 if an error occurred.
 */
int destroy_board(int rows, int** board);

/**
 * @brief Converts a board cell state to a printable character.
 *
 * @param state The state to convert.
 * @return A character representing the state (e.g., ' ', 'X', 'O').
 */
char transform_state_to_char(enum BOARD_STATE state);

#endif