#ifndef _COMPUTER_H
#define _COMPUTER_H

#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>

#include "computer.h"
#include "../game/game.h"
#include "../board/board.h"

/**
 * @brief Selects a random valid column for the AI to play.
 *
 * @param number_of_columns The total number of columns on the board.
 * @param number_of_rows The total number of rows on the board.
 * @param board The current game board.
 * @return The index of a randomly selected column.
 */
int select_random_column(int number_of_columns, int number_of_rows, int** board);

/**
 * @brief Selects a smart column for the AI to play, based on a strategy.
 *
 * @param number_of_columns The total number of columns on the board.
 * @param number_of_rows The total number of rows on the board.
 * @param board The current game board.
 * @return The index of the selected column.
 */
int select_smart_column(int number_of_columns, int number_of_rows, int** board);

/**
 * @brief Checks whether a column is full (i.e., no empty slots).
 *
 * @param col The column index to check. starting from 0.
 * @param rows The number of rows in the board.
 * @param board The current game board.
 * @return true if the column is full, false otherwise.
 */
bool is_column_full(int col, int rows, int** board);

#endif
