#include "test_computer.h"

void test_is_column_full() {
    int rows = 6, cols = 7;
    int** board = setup_board(rows, cols);

    // An empty column should not be considered full
    assert(is_column_full(1, rows, board) == false && "test_is_column_full: FAILED - column should not be full\n");

    // Fill a column completely and test again
    for (int i = 0; i < rows; i++) {
        board[i][0] = PLAYER1;
    }
    assert(is_column_full(0, rows, board) == true && "test_is_column_full: FAILED - column should be full\n");

    destroy_board(rows, board);
    PRINT_SUCCESS("test_is_column_full: PASSED");
}

void test_select_smart_column() {
    int rows = 6, cols = 7;
    int** board = setup_board(rows, cols);

    // Simulate a situation where the AI can win immediately
    board[5][0] = PLAYER2;
    board[4][0] = PLAYER2;
    board[3][0] = PLAYER2;

    assert(select_smart_column(cols, rows, board) == 0 && "test_select_smart_column: FAILED - AI should select column 0\n");
    reset_board(rows, cols, board);

    // Simulate a situation where the player can win on the next move
    board[5][1] = PLAYER1;
    board[4][1] = PLAYER1;
    board[3][1] = PLAYER1;
    assert(select_smart_column(cols, rows, board) == 1 && "test_select_smart_column: FAILED - Player should select column 1\n");

    destroy_board(rows, board);
    PRINT_SUCCESS("test_select_smart_column: PASSED");
}

void test_select_random_column() {
    int rows = 6, cols = 7;
    int** board = setup_board(rows, cols);

    // Select a random column on an empty board
    int col = select_random_column(cols, rows, board);

    assert(col == 1 && "test_select_random_column: FAILED - Expected column 1\n");

    // Fill all columns except one
    for (int j = 0; j < cols - 1; j++) {
        for (int i = 0; i < rows; i++) {
            board[i][j] = PLAYER1;
        }
    }

    // The AI should select the only non-full column, which is column 7 (the last one)
    col = select_random_column(cols, rows, board);
    assert(col == 6 && "test_select_random_column: FAILED - Expected column 6\n");

    destroy_board(rows, board);
    PRINT_SUCCESS("test_select_random_column: PASSED");
}
