#include "game.h"

void place_token(int player, struct coordinate coordinate, int** board) {
    if (coordinate.x != -1 && coordinate.y != -1) {
        board[coordinate.x][coordinate.y] = player == 0 ? PLAYER1 : PLAYER2;
    }
}

struct coordinate find_empty_slot(int x, int rows, int** board) {
    struct coordinate coordinate = { -1, -1 };
    if (x < 0 || x > rows) {
        return coordinate;
    }
    for (int i = rows - 1; i >= 0; i--) {
        if (board[i][x] == EMPTY) {
            coordinate.x = i;
            coordinate.y = x;
            break;
        }
    }
    return coordinate;
}

bool is_valid_column(int column, int col_count) {
    return column >= 0 && column < col_count;
}

int get_player_input() {
    int column;
    printf("\nColumn number? (starts at 1):");
    scanf("%d", &column);
    return column - 1; // Adjust for 0-based indexing
}

bool is_board_full(int current_turn, int row_count, int col_count) {
    return current_turn + 1 == row_count * col_count;
}

int get_selected_column(enum GAME_MODE game_mode, int current_turn, int col_count, int row_count, int** board) {
    if (is_human_turn(game_mode, current_turn)) {
        return get_player_input();
    }
    return get_ai_move(game_mode, col_count, row_count, board);
}

bool is_human_turn(enum GAME_MODE game_mode, int current_turn) {
    return game_mode == PLAYER_VS_PLAYER || current_turn % 2 == 0;
}

int get_ai_move(enum GAME_MODE game_mode, int col_count, int row_count, int** board) {
    switch (game_mode) {
    case PLAYER_VS_RANDOM_AI:
        return select_random_column(col_count, row_count, board);
    case PLAYER_VS_SMART_AI:
        return select_smart_column(col_count, row_count, board);
    default:
        return -1;
    }
}

bool does_player_win(struct coordinate coordinate, int rows, int cols, int** board) {
    if (coordinate.x < 0 || coordinate.y < 0) {
        return false;
    }

    int current_piece = board[coordinate.x][coordinate.y];
    for (int i = 0; i < rows - 3; i++) {

        if (board[i][coordinate.y] == current_piece
            && board[i + 1][coordinate.y] == current_piece
            && board[i + 2][coordinate.y] == current_piece
            && board[i + 3][coordinate.y] == current_piece)
        {
            return true;
        }

    }

    for (int i = 0; i < cols - 3; i++) {
        if (board[coordinate.x][i] == current_piece
            && board[coordinate.x][i + 1] == current_piece
            && board[coordinate.x][i + 2] == current_piece
            && board[coordinate.x][i + 3] == current_piece) {
            return true;
        }
    }

    // Diagonale gauche à droite
    for (int i = 0; i < rows - 3; i++) {
        for (int j = 0; j < cols - 3; j++) {
            if (board[i][j] == current_piece &&
                board[i + 1][j + 1] == current_piece &&
                board[i + 2][j + 2] == current_piece &&
                board[i + 3][j + 3] == current_piece) {
                return true;
            }
        }
    }

    // Diagonale droite à gauche
    for (int i = 0; i < rows - 3; i++) {
        for (int j = 3; j < cols; j++) {
            if (board[i][j] == current_piece &&
                board[i + 1][j - 1] == current_piece &&
                board[i + 2][j - 2] == current_piece &&
                board[i + 3][j - 3] == current_piece) {
                return true;
            }
        }
    }
    return false;
}