#include "test_game.h"

void test_place_token() {
    int rows = 6, cols = 7;
    int** board = setup_board(rows, cols);

    // Player1
    struct coordinate coord = { 2, 3 };
    place_token(0, coord, board);
    assert(board[2][3] == PLAYER1 && "test_place_token: FAILED - Expected PLAYER1 at board[2][3]\n");

    // Player2
    coord.x = 4;
    coord.y = 5;
    place_token(1, coord, board);
    assert(board[4][5] == PLAYER2 && "test_place_token: FAILED - Expected PLAYER2 at board[4][5]\n");

    destroy_board(rows, board);
    PRINT_SUCCESS("test_place_token: PASSED");
}

void test_find_empty_slot() {
    int rows = 6, cols = 7;
    int** board = setup_board(rows, cols);

    // Search for an empty slot in a column
    board[5][2] = PLAYER1;
    struct coordinate coord = find_empty_slot(2, rows, board);
    assert(coord.x == 4 && coord.y == 2 && "test_find_empty_slot: FAILED - Expected coord (4, 2)\n");

    // Search for an empty slot in a column with multiple tokens
    board[4][2] = PLAYER2;
    coord = find_empty_slot(2, rows, board);
    assert(coord.x == 3 && coord.y == 2 && "test_find_empty_slot: FAILED - Expected coord (3, 2)\n");

    destroy_board(rows, board);
    PRINT_SUCCESS("test_find_empty_slot: PASSED");
}

void test_is_valid_column() {
    int cols = 7;

    assert(is_valid_column(0, cols) == true && "test_is_valid_column: FAILED - Column 1 should be valid\n");
    assert(is_valid_column(6, cols) == true && "test_is_valid_column: FAILED - Column 7 should be valid\n");
    assert(is_valid_column(-1, cols) == false && "test_is_valid_column: FAILED - Column 0 should be invalid\n");
    assert(is_valid_column(7, cols) == false && "test_is_valid_column: FAILED - Column 8 should be invalid\n");
    PRINT_SUCCESS("test_is_valid_column: PASSED");
}

void test_is_board_full() {
    int rows = 6, cols = 7;

    assert(is_board_full(41, rows, cols) == true && "test_is_board_full: FAILED - Board should be full\n");
    assert(is_board_full(40, rows, cols) == false && "test_is_board_full: FAILED - Board should not be full\n");
    PRINT_SUCCESS("test_is_board_full: PASSED");
}

void test_does_player_win() {
    int rows = 6, cols = 7;
    int** board = setup_board(rows, cols);

    // Test horizontal win
    board[5][0] = PLAYER1;
    board[5][1] = PLAYER1;
    board[5][2] = PLAYER1;
    board[5][3] = PLAYER1;
    struct coordinate coord = { 5, 3 };
    assert(does_player_win(coord, rows, cols, board) == true && "test_does_player_win: FAILED - Horizontal win should be detected\n");

    // Test vertical win
    fill_board(rows, cols, board);
    board[5][2] = PLAYER2;
    board[4][2] = PLAYER2;
    board[3][2] = PLAYER2;
    board[2][2] = PLAYER2;
    coord.x = 2;
    coord.y = 2;
    assert(does_player_win(coord, rows, cols, board) == true && "test_does_player_win: FAILED - Vertical win should be detected\n");

    // Test diagonal win (left to right)
    fill_board(rows, cols, board);
    board[5][0] = PLAYER1;
    board[4][1] = PLAYER1;
    board[3][2] = PLAYER1;
    board[2][3] = PLAYER1;
    coord.x = 2;
    coord.y = 3;
    assert(does_player_win(coord, rows, cols, board) == true && "test_does_player_win: FAILED - Left-to-right diagonal win should be detected\n");

    // Test diagonal win (right to left)
    fill_board(rows, cols, board);
    board[5][3] = PLAYER2;
    board[4][2] = PLAYER2;
    board[3][1] = PLAYER2;
    board[2][0] = PLAYER2;
    coord.x = 2;
    coord.y = 0;
    assert(does_player_win(coord, rows, cols, board) == true && "test_does_player_win: FAILED - Right-to-left diagonal win should be detected\n");

    destroy_board(rows, board);
    PRINT_SUCCESS("test_does_player_win: PASSED");
}
